﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Threading.Tasks;
using Microsoft.AspNetCore.Authorization;
using Microsoft.AspNetCore.Http;
using Microsoft.AspNetCore.Mvc;
using Tsi1.BusinessLayer.Dtos;
using Tsi1.BusinessLayer.Helpers;
using Tsi1.BusinessLayer.Interfaces;

namespace Tsi1.Api.Controllers
{
    [Route("api/[controller]")]
    [ApiController]
    public class PostController : ControllerBase
    {
        private readonly IPostService _postService;

        private readonly IForumService _forumService;

        private readonly IEmailService _emailService;

        public PostController(IPostService postService, IForumService forumService,
            IEmailService emailService)
        {
            _postService = postService;
            _forumService = forumService;
            _emailService = emailService;
        }


        [Authorize(Roles = UserTypes.Student + ", " + UserTypes.Professor)]
        [HttpGet("GetPosts/{forumId}")]
        public async Task<IActionResult> GetPosts(int forumId)
        {
            var result = await _postService.GetPosts(forumId);

            if (result.HasError)
            {
                return BadRequest(result.Message);
            }

            return Ok(result.Data);
        }

        [Authorize(Roles = UserTypes.Student + ", " + UserTypes.Professor)]
        [HttpPost("Create")]
        public async Task<IActionResult> Create(PostCreateDto newPost)
        {
            var userId = int.Parse(HttpContext.User.Claims.FirstOrDefault(x => x.Type == "Id").Value);

            newPost.UserId = userId;

            var result = await _postService.Create(newPost);

            if (result.HasError)
            {
                return BadRequest(result.Message);
            }

            var userEmails = await _forumService.GetSubscribedUsers(newPost.ForumId);

            if (userEmails.Data.Any())
            {
                var resultEmail = await _emailService.NotifyNewPostOrMessage(newPost, userEmails.Data);

                if (resultEmail.HasError)
                {
                    return BadRequest(resultEmail.Message);
                }
            }
            
            return Ok(result.Data);
        }

        [Authorize(Roles = UserTypes.Student + ", " + UserTypes.Professor)]
        [HttpDelete("Delete/{postId}")]
        public async Task<IActionResult> Delete(int postId)
        {
            var result = await _postService.Delete(postId);

            if (result.HasError)
            {
                return BadRequest(result.Message);
            }

            return Ok();
        }
    }
}
