﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Threading.Tasks;
using Microsoft.AspNetCore.Authorization;
using Microsoft.AspNetCore.Http;
using Microsoft.AspNetCore.Mvc;
using Tsi1.BusinessLayer.Dtos;
using Tsi1.BusinessLayer.Helpers;
using Tsi1.BusinessLayer.Interfaces;

namespace Tsi1.Api.Controllers
{
    [Route("api/[controller]")]
    [ApiController]
    public class PostMessageController : ControllerBase
    {
        private readonly IPostMessageService _postMessageService;

        public PostMessageController(IPostMessageService postMessageService)
        {
            _postMessageService = postMessageService;
        }

        [Authorize(Roles = UserTypes.Student + ", " + UserTypes.Professor)]
        [HttpGet("GetPostMessages/{postId}")]
        public async Task<IActionResult> GetPostMessagesPosts(int postId)
        {
            var result = await _postMessageService.GetPostMessages(postId);

            if (result.HasError)
            {
                return BadRequest(result.Message);
            }

            return Ok(result.Data);
        }

        [Authorize(Roles = UserTypes.Student + ", " + UserTypes.Professor)]
        [HttpPost("Create")]
        public async Task<IActionResult> Create(PostMessageCreateDto newPostMessage)
        {
            var userId = int.Parse(HttpContext.User.Claims.FirstOrDefault(x => x.Type == "Id").Value);

            newPostMessage.UserId = userId;

            var result = await _postMessageService.Create(newPostMessage);

            if (result.HasError)
            {
                return BadRequest(result.Message);
            }

            return Ok(result.Data);
        }

        [Authorize(Roles = UserTypes.Student + ", " + UserTypes.Professor)]
        [HttpDelete("Delete/{postMessageId}")]
        public async Task<IActionResult> Delete(int postMessageId)
        {
            var result = await _postMessageService.Delete(postMessageId);

            if (result.HasError)
            {
                return BadRequest(result.Message);
            }

            return Ok();
        }
    }
}
