﻿using System.Collections.Generic;
using System.Threading.Tasks;
using Microsoft.AspNetCore.Authorization;
using Microsoft.AspNetCore.Mvc;
using Tsi1.BusinessLayer.Dtos;
using Tsi1.BusinessLayer.Helpers;
using Tsi1.BusinessLayer.Interfaces;

namespace Tsi1.Api.Controllers
{
    [Authorize(Roles = UserTypes.Professor)]
    [Route("api/[controller]")]
    [ApiController]
    public class SectionController : ControllerBase
    {
        private readonly ISectionService _sectionService;

        public SectionController(ISectionService sectionService)
        {
            _sectionService = sectionService;
        }

        [HttpPost("Create")]
        public async Task<IActionResult> Create(SectionCreateDto section)
        {
            var result = await _sectionService.Create(section);
            if (result.HasError)
            {
                return BadRequest(result.Message);
            }

            return Ok(result.Data);
        }

        [HttpPut("Modify/{sectionId}")]
        public async Task<IActionResult> Modify(int sectionId, string newName)
        {
            var result = await _sectionService.Modify(sectionId, newName);
            if (result.HasError)
            {
                return BadRequest(result.Message);
            }

            return Ok();
        }

        [HttpDelete("Delete/{sectionId}")]
        public async Task<IActionResult> Delete(int sectionId)
        {
            var result = await _sectionService.Delete(sectionId);
            if (result.HasError)
            {
                return NotFound(result.Message);
            }

            return Ok();
        }

        [HttpPost("ChangeOrder")]
        public async Task<IActionResult> ChangeOrder(List<OrderDto> orderDtos)
        {
            var result = await _sectionService.OrderSections(orderDtos);
            if (result.HasError)
            {
                return BadRequest(result.Message);
            }

            return Ok();
        }
    }
}
